"use strict";
/*
 * @author KaySaith
 * @date 2021/4/21
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventQueryHelper = void 0;
const live_data_1 = require("@towify-types/live-data");
const soid_data_1 = require("soid-data");
const event_query_value_1 = require("../type/event.query.value");
class EventQueryHelper {
    static convertToLiveDataEngineQueryType(queries) {
        return (queries.map(query => ({
            fieldPath: query.path,
            value: query.condition.content.value || '',
            condition: query.condition.type
        })) || []);
    }
    /** 转化成 Scf 需要的 Parameter 的入参数格式 */
    static convertToScfFindContent(params) {
        var _a, _b, _c;
        const queries = [];
        (_b = (_a = params.executor.queries) === null || _a === void 0 ? void 0 : _a.filter(query => query.condition !== live_data_1.QueryConditionEnum.All)) === null || _b === void 0 ? void 0 : _b.forEach(query => {
            queries.push(this.convertToQueryJson(params.tableHashName, query));
        });
        const queryFilter = {};
        // 如果没有设置 Query 那么默认为查找整个表格的数据
        if (queries.length === 0) {
            queryFilter.query = { tableId: params.tableHashName, filter: {} };
        }
        EventQueryHelper.mergeDeep(queryFilter, ...queries);
        return {
            precondition: {
                needToken: ((_c = params.preconditions) === null || _c === void 0 ? void 0 : _c.needToken) === true,
                queries: []
            },
            executor: {
                query: queryFilter.query,
                sort: params.executor.sort,
                limit: params.executor.limit,
                skip: params.executor.skip
            }
        };
    }
    /** 将多条 Query 的 Json 进行深度去重合并 */
    static mergeDeep(target, ...sources) {
        if (!sources.length)
            return target;
        const source = sources.shift();
        if (source && soid_data_1.ObjectUtils.isObject(target) && soid_data_1.ObjectUtils.isObject(source)) {
            Object.keys(source).forEach(key => {
                if (soid_data_1.ObjectUtils.isObject(source[key])) {
                    if (!target[key])
                        Object.assign(target, { [key]: {} });
                    this.mergeDeep(target[key], source[key]);
                }
                else {
                    Object.assign(target, { [key]: source[key] });
                }
            });
        }
        return this.mergeDeep(target, ...sources);
    }
    static quote(content) {
        return `\\Q${content.replace('\\E', '\\E\\\\E\\Q')}\\E`;
    }
    static convertToQueryJson(initialTableHashName, query) {
        const convertToSql = (condition, value) => {
            const content = {};
            switch (condition) {
                case live_data_1.QueryConditionEnum.IsNotEmpty:
                case live_data_1.QueryConditionEnum.IsEmpty: {
                    content[event_query_value_1.sqlKeyWorld[condition]] =
                        condition !== live_data_1.QueryConditionEnum.IsEmpty;
                    break;
                }
                case live_data_1.QueryConditionEnum.StartWith: {
                    content[event_query_value_1.sqlKeyWorld[condition]] = `^${EventQueryHelper.quote(value)}`;
                    break;
                }
                case live_data_1.QueryConditionEnum.EndWith: {
                    content[event_query_value_1.sqlKeyWorld[condition]] = `${EventQueryHelper.quote(value)}$`;
                    break;
                }
                case live_data_1.QueryConditionEnum.DoesNotContain:
                case live_data_1.QueryConditionEnum.ContainsAll:
                case live_data_1.QueryConditionEnum.ContainsIn: {
                    // todo 接受数组参数
                    content[event_query_value_1.sqlKeyWorld[condition]] = [value];
                    break;
                }
                case live_data_1.QueryConditionEnum.Contains: {
                    content[event_query_value_1.sqlKeyWorld[condition]] = EventQueryHelper.quote(value);
                    break;
                }
                default: {
                    content[event_query_value_1.sqlKeyWorld[condition]] = value;
                }
            }
            return content;
        };
        const content = convertToSql(query.condition, query.value);
        const getQueryDetail = (path) => {
            const result = {};
            result[path.fieldHashName] = { tableId: path.referenceTableHashName };
            if (path.reference) {
                result[path.fieldHashName].filter = getQueryDetail(path.reference);
            }
            else {
                result[path.fieldHashName] = content;
            }
            return result;
        };
        return {
            query: {
                tableId: initialTableHashName,
                filter: Object.assign({}, getQueryDetail(query.fieldPath))
            }
        };
    }
}
exports.EventQueryHelper = EventQueryHelper;
